/**
 *
 * Performs background subtraction on a set of inputted BufferedImage objects.
 * This is actually not used but left in for completion and may be useful in the future.
 * 
 * Main changes from original is removal of several stored BufferedImage objects, which were
 * pointless, and even more so since I am not using this at all.
 *
 * Original Code: David Bull - http://www.uk-dave.com/projects/java/colourtracker.php
 * Modified by: Scott Culcheth
 *
 * Further modification: Andrew Armstrong
 * Last Modified: 04.05.08
 *
 */
import java.util.*;
import java.awt.*;
import java.awt.image.*;

public class BackgroundSubtracter
{
	// Variables for what we alter the final image to be - empty is black, full is green.
	public static final int EMPTY_REGION = 0x000000;
	public static final int FULL_REGION  = 0x00FF00;

	// Default variance set here. This is altered via. a GUI slider too.
	private int variance = 25;

	// For logging
	private CarAIMain parentCarAIMain;

	private Vector imageListeners = new Vector();

	public BackgroundSubtracter(CarAIMain passedCarAIMain)
	{
		parentCarAIMain = passedCarAIMain;
	}

	// Get/Set threshold values (using a 0-255 slider).
	public int getThreshold()
	{
		return variance;
	}
	public void setThreshold(int newT)
	{
		variance = newT;
	}

	// Applies the background subtraction on a specific point x, y using images liveImage and referenceImage references.
	// Returns FULL_REGION or EMPTY_REGION.
	public int applyBackgroundSubtraction(int x, int y, BufferedImage liveImage, BufferedImage referenceImage)
	{
		int rgbLive=0, rgbLast=0, rLive=0, rLast=0, gLive=0, gLast=0, bLive=0, bLast=0;

		rgbLive = liveImage.getRGB(x,y);
		rgbLast = referenceImage.getRGB(x,y);

		rLive = (rgbLive & 0xFF0000) >> 16;
		gLive = (rgbLive & 0x00FF00) >> 8;
		bLive = (rgbLive & 0x0000FF);

		rLast = (rgbLast & 0xFF0000) >> 16;
		gLast = (rgbLast & 0x00FF00) >> 8;
		bLast = (rgbLast & 0x0000FF);

		parentCarAIMain.log("RLIVE: "+rLive+" RLAST: "+rLast+" GLIVE:"+gLive+" GLAST: "+gLast+" BLIVE:"+bLive+" BLAST: "+bLast);

		if(!within(rLive, rLast, variance) && !within(gLive, gLast, variance) && !within(bLive, bLast, variance))
		{
			return FULL_REGION;
		}
		else
		{
			return EMPTY_REGION;
		}
	}
	
	// Apply background subtraction to the images liveImage and referenceImage.
	// Returns the final background subtracted image
	public BufferedImage applyBackgroundSubtraction(BufferedImage liveImage, BufferedImage referenceImage)
	{
		BufferedImage differenceFrame = new BufferedImage(liveImage.getWidth(), liveImage.getHeight(), BufferedImage.TYPE_INT_RGB);
	
		// Code below unaltered from Scott's code.
		int rgbLive=0, rgbLast=0, rLive=0, rLast=0, gLive=0, gLast=0, bLive=0, bLast=0;

		for (int y=0; y<liveImage.getHeight(); y++)
		{
			for (int x=0; x<liveImage.getWidth(); x++)
			{
				rgbLive = liveImage.getRGB(x,y);
				rgbLast = referenceImage.getRGB(x,y);

				rLive = (rgbLive & 0xFF0000) >> 16;
				gLive = (rgbLive & 0x00FF00) >> 8;
				bLive = (rgbLive & 0x0000FF);

				rLast = (rgbLast & 0xFF0000) >> 16;
				gLast = (rgbLast & 0x00FF00) >> 8;
				bLast = (rgbLast & 0x0000FF);

				if(!within(rLive, rLast, variance) && !within(gLive, gLast, variance) && !within(bLive, bLast, variance))
				{
					// Green
					differenceFrame.setRGB (x, y, FULL_REGION);
				}
				/*else
				{
					// Black
					// Not required since the BufferedImage is by default black
					differenceFrame.setRGB (x, y, EMPTY_REGION);
				}*/
			}
		}
		// Return the final image
		return differenceFrame;
	}

	// Simple absolute-using within function
	private boolean within(int a, int b, int x)
	{
		if((Math.abs(a-b))<x)
		{
			return true;
		}
		return false;
	}

}
