/**
 * CameraImage.java
 *
 * Image sent from camera, instantiated by CarAIMain.java and placed inside a JImagePanel object.
 * Commented-out segments remnants of image processing techniques of CarDriverGUI.java (Culcheth, 2006) (Now removed: AA)
 *
 * This now simply holds the image we have for our live camera, which is put up on the GUI - can be manipulated by other classes to get
 * the current image. Turns the camera on and off too.
 *
 * Original Code: David Bull - http://www.uk-dave.com/projects/java/colourtracker.php
 * Modified by: Scott Culcheth
 * Further modified (from CarDriverGUI.java) by: Chris Carter
 * Further modification by: Andrew Armstrong
 * Last Modified: 04.05.08
 *
 */

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;

public class CameraImage extends JPanel implements ImageUpdateListener{

	////////////////////////////////////////////////////
	// CONSTANTS AND CLASS VARIABLES
	////////////////////////////////////////////////////

	// liveImage is now public so the image can be used	
	public JImagePanel liveImage;
	public ImageGrabber imageGrabber;
	public CameraController cameraController;
	private GridBagLayout gridBag;

	public static final String CAPTURE_DEVICE_NAME = "vfw:Microsoft WDM Image Capture (Win32):2";
	public static final String CAPTURE_DEVICE_LOCATION = "vfw://0";

	private final static long serialVersionUID = 1L;

	////////////////////////////////////////////////////
	// CONSTURCTOR
	////////////////////////////////////////////////////

	/**
	 * Creates a new Colour Tracker GUI.
	 * @param app the Colour Tracker application.
	 */
	public CameraImage() {
		gridBag = new GridBagLayout();
		this.setLayout(gridBag);

	//**************
	//* Live Image *
	//**************

		liveImage = new JImagePanel();
		this.add(liveImage);
	}
	
	////////////////////////////////////////////////////
	// METHODS
	////////////////////////////////////////////////////

	// Turns the camera on, report errors to sim.infoPanel
	public void enableCam(CarAIMain passedCarAIMain) {
		try{
			// Note: Any logging after "CameraController" seem to randomly make Java crash		
			// ...or not...it's randomly crashing!
			cameraController = new CameraController(CAPTURE_DEVICE_NAME, CAPTURE_DEVICE_LOCATION, new Dimension(320, 240), passedCarAIMain);
			imageGrabber = new ImageGrabber(cameraController);

			liveImage.setImage(imageGrabber.getLiveImage());
			gridBag.setConstraints(liveImage, new GridBagConstraints(0,1,3,7,0,0,GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(0,0,0,0),0,0));

			imageGrabber.addImageUpdateListener(this);
			liveImage.camOn = true;
		}
		catch(Exception e){
			passedCarAIMain.log("Error initialising camera! Error: " + e.toString());
		}
	}

	// Turns the camera off
	public void disableCam(){
		imageGrabber.shutdown();
		cameraController.shutdown();
		liveImage.camOn = false;
		liveImage.noImage();
	}

	public void imagesUpdated(){
		liveImage.setImage(imageGrabber.getLiveImage());
	}

	// Returns the latest image
	public BufferedImage getLiveImage() {
		if(liveImage.camOn)
		{
			imagesUpdated();
		}
		return liveImage.getImage();
	}
}
