/**
 * Frame differencing
 *
 * This causes 2 images inputted (usually a past frame and the current frame) are differenced, so different pixels are noted.
 * 
 * There really is only a need for static functions and no need to initalise this. The code is modified from the background subtraction code.
 * 
 * Created by: Andrew Armstrong
 * Last Modified: 04/05/08
 *
 */

import java.util.*;
import java.awt.*;
import java.awt.image.*;

public class FrameDifferencing
{
	public static final int EMPTY_REGION = 0x000000;
	public static final int FULL_REGION  = 0x00FF00;

	// Does the FrameDifferencing between image1 and image2
	// Useful when both are thresholded first
	// Can be done between 2 arbitarily sized images but they need to be the same size.
	public static BufferedImage frameDifference(BufferedImage image1, BufferedImage image2) { return frameDifference(image1, image2, 0, image1.getWidth(), 0, image1.getHeight()); }
	public static BufferedImage frameDifference(BufferedImage image1, BufferedImage image2, int minX, int maxX, int minY, int maxY) {
		// New image to return
		BufferedImage returnImage = new BufferedImage(image1.getWidth(), image1.getHeight(), BufferedImage.TYPE_INT_RGB);
		
		// Loop the pixels
		for (int x = minX; x < maxX; x++)
		{
			for (int y = minY; y < maxY; y++)
			{
				// There must be a difference to paint a FULL pixel
				if(image1.getRGB(x,y) != image2.getRGB(x,y))
				{
					returnImage.setRGB(x, y, FULL_REGION);
				}				
				/* // Else we print a EMPTY pixel
				// This isn't required, since the "new BufferedImage()" creates an entirely black image by default.
				else
				{
					returnImage.setRGB(x, y, EMPTY_REGION);					
				}*/
			}
		}
		// Return the completed image
		return returnImage;
	}

	// Does frame differencing directly on image2, instead of creating a new image to return
	public static void frameDifferenceDirectly(BufferedImage image1, BufferedImage image2) { frameDifferenceDirectly(image1, image2, 0, image1.getWidth(), 0, image1.getHeight()); }
	public static void frameDifferenceDirectly(BufferedImage image1, BufferedImage image2, int minX, int maxX, int minY, int maxY) {
		// Loop the pixels
		for (int x = minX; x < maxX; x++)
		{
			for (int y = minY; y < maxY; y++)
			{
				int image1stuff = image1.getRGB(x,y);
				int image2stuff = image2.getRGB(x,y);
			
				// There must be a difference to paint a FULL pixel
				if(image1stuff != image2stuff)
				{
					image2.setRGB(x, y, FULL_REGION);
				}				
				// Else we print a EMPTY pixel
				// This is needed - image2 might be some old "live" image
				else
				{
					image2.setRGB(x, y, EMPTY_REGION);					
				}
			}
		}
	}
}