/**
 * ImageGrabber.java
 *
 * Gets image from camera. Called by CameraImage.java, for use with Simulation.java
 *
 * Original Code: David Bull - http://www.uk-dave.com/projects/java/colourtracker.php
 * Modified by: Scott Culcheth
 * Further modified by: Chris Carter
 *
 * Last Modified: 19.04.07
 *
 */
import java.util.*;
import java.awt.*;
import java.awt.image.*;
import java.lang.*;

public class ImageGrabber
{
	////////////////////////////////////////////////////
	// CONSTANTS AND CLASS VARIABLES
	////////////////////////////////////////////////////

	public int frameGrabFrequency;

//	private CarDriver app;
	private CameraController camCont;

	private BufferedImage liveImage;
	private VideoUpdateThread videoUpdateThread;

//	private Vector<ImageUpdateListener> imageListeners = new Vector<ImageUpdateListener>();
	private Vector imageListeners = new Vector();

	////////////////////////////////////////////////////
	// CONSTRUCTOR
	////////////////////////////////////////////////////

	/**
	 * Creates a new Image Processor.
	 * @param cC the Camera Controller.
	 */
	public ImageGrabber(CameraController cC){

		camCont = cC;
		liveImage = camCont.getFrame();

		frameGrabFrequency = 1000 / ((int)(camCont.getMaxFrameRate()));

		videoUpdateThread = new VideoUpdateThread();
		videoUpdateThread.start();

	}


	////////////////////////////////////////////////////
	// METHODS
	////////////////////////////////////////////////////

	/**
	 * Shutsdown the Video Update Thread.
	 */
	public void shutdown()
	{
		videoUpdateThread.shutdown();
	}


	/**
	 * Returns the live image from the video capture devive.
	 * @return live image from the video capture devive.
	 */
	public BufferedImage getLiveImage()
	{
		synchronized (liveImage)
		{
			return liveImage;
		}
	}

	////////////////////////////////////////////////////
	// LISTENER METHODS
	////////////////////////////////////////////////////

	public void addImageUpdateListener(ImageUpdateListener listener) { imageListeners.add(listener); }

	public void removeImageUpdateListener(ImageUpdateListener listener) { imageListeners.remove(listener); }

	private void fireImageUpdateEvent() {
			for (int i=0; i<imageListeners.size(); i++)
				((ImageUpdateListener)imageListeners.elementAt(i)).imagesUpdated();
	}

	////////////////////////////////////////////////////
	// INNER-CLASSES
	////////////////////////////////////////////////////

	/**
	 * Thread to grab images from the video capture device and process the image for colour tracking.
	 */
	private class VideoUpdateThread extends Thread
	{
		private boolean running = false;
		public VideoUpdateThread() {}
		public void run()
		{
			running=true;
			while (running)
			{
				synchronized (liveImage)
				{
					liveImage = camCont.getFrame();
				}
				fireImageUpdateEvent();
				try { sleep(frameGrabFrequency); }
				catch (InterruptedException e) {
					// No logging in this class
				}
			}
		}
		public void shutdown() { running=false; }
	}
}
