/**
 * SimAction.java
 * Defines an action to be taken. Objects generated by SimActionHandler.java,
 * Called by SimControl.java, for use with DrawSim.java.
 *
 * Modified By: Andrew Armstrong: Renamed to SimAction from Action
 *
 * Created by: Chris Carter
 * Last Modified: 04.05.08
 *
 */

import java.awt.*;

public final class SimAction {

	double pulseCoeff;
	double pulses;
	SimCar car;
	double refsPerPulse;
	double thetaInc, fractThetaInc;


	boolean noMoreInsts = false;//Boolean indicating whether any more instructions remain
	boolean error = false;//Boolean indicating whether an error has occurred

	boolean complete = false;//Boolean indicating whether this current action has completed or not

	int type;
	//Type Key: 1:F, 2:FR, 3:FL, 4:B, 5:BR, 6:BL

	//Constructor to generate actions
	public SimAction(SimCar passedCar, int puls, double pCoeff, int typ){

		car = passedCar;
		pulseCoeff = pCoeff;
		type = typ;
		pulses = puls;
		calcRefsPerPulse();
		calcThetaInc();

	}


	public SimAction(boolean flag){//Dummy action to indicate no more instructions left to process, or error
		if (flag){
			noMoreInsts = true;
			type = 0;
		}
		else {
			error = true;
			type = -1;
		}

	}

	//Calculates how frequently drive power needs sending to car
	private void calcRefsPerPulse(){

		double distPerPulse;

		switch(type){

			case 1: 	//if(pulseCoeff >= 1){
							distPerPulse = car.UNIT_STRGHT_SPEED;// * pulseCoeff;
						//} else {
						//	distPerPulse = car.UNIT_STRGHT_SPEED / (2 - pulseCoeff);
						//}
						refsPerPulse = distPerPulse / car.REFRATE;
						break;

			case 2:		//if(pulseCoeff >= 1){
							distPerPulse = car.R_UNIT_TURN_SPEED;// * pulseCoeff;
						//} else {
						//	distPerPulse = car.R_UNIT_TURN_SPEED / (2 - pulseCoeff);
						//}
						refsPerPulse = distPerPulse / car.REFRATE;
						break;

			case 3:		//if(pulseCoeff >= 1){
							distPerPulse = car.L_UNIT_TURN_SPEED;// * pulseCoeff;
						//} else {
						//	distPerPulse = car.L_UNIT_TURN_SPEED / (2 - pulseCoeff);
						//}
						refsPerPulse = distPerPulse / car.REFRATE;
						break;

			case 4:		//CODE ONCE REVERSE MEASUREMENTS ARE KNOWN
						break;

			case 5:		//CODE ONCE REVERSE MEASUREMENTS ARE KNOWN
						break;

			case 6:		//CODE ONCE REVERSE MEASUREMENTS ARE KNOWN
						break;

			default:	error = true;
						break;

		}

	}

	//Calculates the required theta increment per refresh of the simulation (turning action)
	private void calcThetaInc(){

		double anglePerPulse;

		//Calculate thetaInc for the complete Refs
		switch(type){

			case 1:		//if(pulseCoeff >= 1){
							anglePerPulse = (Math.PI * pulseCoeff) / car.UNIT_STRGHT_VEER;
						//} else {
						//	anglePerPulse = Math.PI / (car.UNIT_STRGHT_VEER * (2 - pulseCoeff));
						//}
						thetaInc = anglePerPulse / getCompleteRefs();
						fractThetaInc = thetaInc * getFractRef();
						break;

			case 2:		//if(pulseCoeff >= 1){
							anglePerPulse = (Math.PI * pulseCoeff) / car.R_UNIT_TURN_RATE;
						//} else {
						//	anglePerPulse = Math.PI / (car.R_UNIT_TURN_RATE * (2 - pulseCoeff));
						//}
						thetaInc = anglePerPulse / getCompleteRefs();
						fractThetaInc = thetaInc * getFractRef();
						break;

			case 3:		//if(pulseCoeff >= 1){
							anglePerPulse = (Math.PI * pulseCoeff) / car. L_UNIT_TURN_RATE;
						//} else {
						//	anglePerPulse = Math.PI / (car.L_UNIT_TURN_RATE * (2 - pulseCoeff));
						//}
						thetaInc = anglePerPulse / getCompleteRefs();
						fractThetaInc = thetaInc * getFractRef();
						break;

			case 4:		//CODE ONCE REVERSE MEASUREMENTS ARE KNOWN
						break;

			case 5:		//CODE ONCE REVERSE MEASUREMENTS ARE KNOWN
						break;

			case 6:		//CODE ONCE REVERSE MEASUREMENTS ARE KNOWN
						break;

			default:	error = true;
						break;

		}

	}

	//Returns number of complete refreshes required
	public int getCompleteRefs(){

		int result;

		result = (int) Math.floor(refsPerPulse);

		return result;

	}

	//Returns the oddment value of the refreshes, thereby giving a fractional refresh required at end of each pulse
	public double getFractRef(){

		double result;

		result = refsPerPulse - Math.floor(refsPerPulse);

		return result;

	}

}