/**
 * SimActionHandler.java
 * Processes information required for next instruction to be executed and generates an Action object.
 * Called by SimDraw.java, for use with CarAIMain.java.
 *
 * Created by: Chris Carter
 * Last Modified: 06.05.07
 *
 */

public class SimActionHandler {

	SimNavigator nav;
	SimControl simControl;
	SimCar car;

	int actCounter;//Number of actions carried out

	public SimActionHandler(String[] endLoc, int numLaps, SimCar passedCar, CarAIMain passedCarAIMain){

		nav = new SimNavigator("inst.txt", passedCarAIMain);
		simControl = new SimControl(endLoc, numLaps);
		car = passedCar;
		actCounter = 0;
	}

	public SimAction nextAction(){

		SimAction action;

		String dir, steer;
		double coeff;
		int pulses;

		dir = nav.getInst();//Get next instruction parameter (should be FW or BW)

		if(dir.matches("EOF")){
			action = new SimAction(true);
		}
		else if (dir.matches("ERROR")){
			action = new SimAction(false);
		}
		else {
			steer = nav.getInst();//Get next instruction parameter (should be L, R or STRGHT)
			if(steer.matches("ERROR")){
				action = new SimAction(false);
			}
			else {
				pulses = (int) nav.getAttribute();//Get next instruction parameter (should be number of pulses)
				if(pulses == 0){
					action = new SimAction(false);
				}
				else {
					coeff = nav.getAttribute();//Get next instruction parameter (should be pulse coefficient)
					if(coeff == 0.0){
						action = new SimAction(false);
					}
					else {
						//Type Key: 1:F, 2:FR, 3:FL, 4:B, 5:BR, 6:BL
						if(dir.matches("FW")){
							if(steer.matches("STRGHT")){
								action = new SimAction(car, pulses, coeff, 1);
							}
							else if(steer.matches("R")){
								action = new SimAction(car, pulses, coeff, 2);
							}
							else if(steer.matches("L")){
								action = new SimAction(car, pulses, coeff, 3);
							}
							else{
								action = new SimAction(false);
							}
						}
						else if (dir.matches("BW")){
							if(steer.matches("STRGHT")){
								action = new SimAction(car, pulses, coeff, 4);
							}
							else if(steer.matches("R")){
								action = new SimAction(car, pulses * -1, coeff, 5);
							}
							else if(steer.matches("L")){
								action = new SimAction(car, pulses, coeff, 6);
							}
							else{
								action = new SimAction(false);
							}
						}
						else {
							action = new SimAction(false);
						}
					}
				}
			}
		}
		return action;

	}

	//Begin execution of passed action
	public void beginNextAction(SimCar car, SimAction action, boolean driveCar){

		simControl.initialiseAction(action);
		actCounter++;
		continueAction(car, action, driveCar);

	}

	//Continue execution of passed action
	public void continueAction(SimCar car, SimAction action, boolean driveCar){

		simControl.runSim(car, action, driveCar);

	}

	public boolean getComplete(){

		return simControl.simComplete;

	}

	public String getPowerInfo(){

		return simControl.powerInfo;

	}

 }