/**
 * SimCar.java
 * Generates a car object. Called by SimDraw.java, referenced by various, for use with CarAIMain.java.
 *
 * Car orientation:
 *
 *					A------B
 *            Rear  	     Front
 *					D------C
 *
 * Modified by: Andrew Armstrong: Renamed to SimCar from Car.
 *
 * Created by: Chris Carter
 * Last Modified: 04.05.08
 *
 */
 
import java.awt.*;
import java.lang.*;


public class SimCar {
/* 
	static int LENGTH = 83;
	static int WIDTH = 38;
	 */
	static int LENGTH = 41;
	static int WIDTH = 19;

	//N.B. Car has steering bias set fully to the right
	//static int R_TURN_RADIUS = 170;//Radius of circle created by car when turning right (full lock)
	//static int L_TURN_RADIUS = 450;//Radius of circle created by car when turning left (full lock)
	//static int R_VEER_RADIUS = 2400;//Radius of circle created by car's 'veer' to the right when no turning is applied

	/**********************************************************
	 * ATTRIBUTES ALTER DEPENDANT ON BATTERIES & CHARGE LEVEL *
	 * CURRENTLY SET FOR: Semi-Fresh Rechargeables            *
	 **********************************************************/

	/* static int UNIT_STRGHT_SPEED = 42;//number of pixels moved forwards per 100ms pulse with no steering
	static double UNIT_STRGHT_VEER = 45;//PI div by this value = radians veered per 100ms pulse with no steering

	static int R_UNIT_TURN_SPEED = 35;//number of pixels moved forwards per 100ms pulse while turning right
	static double R_UNIT_TURN_RATE = 4.2;//PI div by this value = radians turned per 100ms pulse when turning right

	static int L_UNIT_TURN_SPEED = 41;//number of pixels moved forwards per 100ms pulse while turning left
	static double L_UNIT_TURN_RATE = 9.75;//PI div by this value = radians turned per 100ms pulse when turning left */

 	static int UNIT_STRGHT_SPEED = 44;//number of pixels moved forwards per 100ms pulse with no steering
	static double UNIT_STRGHT_VEER = 90;//PI div by this value = radians veered per 100ms pulse with no steering

	static int R_UNIT_TURN_SPEED = 35;//number of pixels moved forwards per 100ms pulse while turning right
	static double R_UNIT_TURN_RATE = 8.4;//PI div by this value = radians turned per 100ms pulse when turning right

	static int L_UNIT_TURN_SPEED = 41;//number of pixels moved forwards per 100ms pulse while turning left
	static double L_UNIT_TURN_RATE = 19.5;//PI div by this value = radians turned per 100ms pulse when turning left 
	
	/* static int UNIT_STRGHT_SPEED = 85;//number of pixels moved forwards per 100ms pulse with no steering
	static double UNIT_STRGHT_VEER = 90;//PI div by this value = radians veered per 100ms pulse with no steering

	static int R_UNIT_TURN_SPEED = 70;//number of pixels moved forwards per 100ms pulse while turning right
	static double R_UNIT_TURN_RATE = 8.4;//PI div by this value = radians turned per 100ms pulse when turning right

	static int L_UNIT_TURN_SPEED = 82;//number of pixels moved forwards per 100ms pulse while turning left
	static double L_UNIT_TURN_RATE = 19.5;//PI div by this value = radians turned per 100ms pulse when turning left  */
	
    //------------------------------------------------------------------------------------------------------------------

	static int REFRATE = 10;//Car's refresh rate/frequency i.e. after how many pixels movement should its position be redrawn

	double thetaX;//Angle with X axis

	static Color COLOR = Color.yellow;

	//4 value arrays of form [A_co-ord, B_co-ord, C_co-ord, D_co-ord]
	int[] xCoords = new int[4];
	int[] yCoords = new int[4];

	SimCar2D carPoly;//Graphical representation of car

	//Constructor to define car's initial position
	public SimCar(int x, int y, double theta){
		//Initial angle with X axis
		thetaX = theta;

		//Point A initial position
		xCoords[0] = x;
		yCoords[0] = y;

		//Point B initial position
		xCoords[1] = (int) Math.round(x + (Math.cos(SimCalcs.calcThetaR(thetaX)) * LENGTH));
		yCoords[1] = (int) Math.round(y + (Math.sin(SimCalcs.calcThetaR(thetaX)) * LENGTH));

		//Point C initial position
		xCoords[2] = (int) Math.round(xCoords[1] - (Math.sin(SimCalcs.calcThetaR(thetaX)) * WIDTH));
		yCoords[2] = (int) Math.round(yCoords[1] + (Math.cos(SimCalcs.calcThetaR(thetaX)) * WIDTH));

		//Point D initial position
		xCoords[3] = (int) Math.round(x - (Math.sin(SimCalcs.calcThetaR(thetaX)) * WIDTH));
		yCoords[3] = (int) Math.round(y + (Math.cos(SimCalcs.calcThetaR(thetaX)) * WIDTH));


		//Generate initial polygon representation
		carPoly = new SimCar2D(xCoords, yCoords);
	}
}