/**
 * SimCarControl.java
 * Actions the commands passed by SimControl.java
 * Called by SimControl.java for use with CarAIMain.java
 *
 *
 * Created by: Chris Carter
 * With imported code segments from:
 *		PWMCarControl.java & PWMThread.java, Scott Culcheth, Loughborough University (2006)
 * 		ParCarControl.java, Simon Lucas, Essex University (downloaded from http://algoval.essex.ac.uk/cec2005/race/Parsoftware.html)
 *
 * Modified by: Andrew Armstrong, renamed to SimCarControl, was previously CarControl.
 *
 * Last Modified: 07.05.07
 *
 */

import parport.ParallelPort;

public class SimCarControl implements StaticControls{

	ParallelPort pp = new ParallelPort(0x378); //0x denotes hex addressing, 0x378 is the hex base address of LPT1

	int leftBit = 0x01; //i.e. data pin 0 of parallel port
	int rightBit = 0x02; //i.e. data pin 1 of parallel port
	int forwardBit = 0x04; //i.e. data pin 2 of parallel port
    int backwardBit = 0x08; //i.e. data pin 3 of parallel port
    //N.B. If 2 cars are to be used, shift above 4 bits to right to access other 4 data pins (for control of second car)

	int control = 0x00; //i.e. all utilised data ports set to 0

	String exitMsg;

	public SimCarControl(){

		pp.write(0x00);//turn all parallel port data pins off for start of sim

	}

	public void drive(int speed, int dir){

		if(speed > 0){ //go forwards

			control = forwardBit;
			if(dir > 0){
				steer(dir);//get turning pulse
			}

		}
		else if(speed < 0){ //go backwards - DOESN'T OCCUR AT PRESENT

			control = backwardBit;
			if(dir > 0){
				steer(dir);//get turning pulse
			}
		}

		else { //send no drive pulse

			control = 0x00;
			if(dir > 0){
				steer(dir);//get turning pulse
			}
		}

		pp.write(control);//send pulse to car

	}

	public void steer(int dir){

		switch(dir) {

			case(LEFT):		control |= leftBit;
							//leftNoPower();
							break;

			case(RIGHT): 	control |= rightBit;
							//rightNoPower();
						 	break;

			default:		break;

		}

	}

	//Turn off all data pins, and report reason
	public void close(int exitCode){

		switch(exitCode) {

			case 1:		exitMsg = "Simulation has finished, stopping car";
						control = 0x00;
						pp.write(control);
						break;

			case 2:		exitMsg = "Simulation paused/halted by user, stopping car...";
						control = 0x00;
						pp.write(control);
						break;

			case 3:		exitMsg = "Car has crashed, stopping simulation";
						control = 0x00;
						pp.write(control);
						break;

			case 4:		exitMsg = "Car has crashed";
						break;

			case 5:		exitMsg = "All instructions executed, stopping simulation";
						control = 0x00;
						pp.write(control);
						break;

			case 6:		exitMsg = "Simulation exited, car stopped";
						control = 0x00;
						pp.write(control);
						break;

			default:	exitMsg = "Bad Instruction - Code error in SimControl.java";
						control = 0x00;
						pp.write(control);
						break;

		}

	}

}