/**
 * SimControl.java
 *
 * Determines calculations required for simulation and calls SimCalcs and SimCarControl classes in order to carry them out
 * SimDraw can then use the modified attributes of the car to refresh the simulation interface
 * Called by SimActionHandler.java for use with Simulation.java
 *
 * Created by: Chris Carter
 * Last Modified: 06.05.07
 *
 */
 
import java.awt.*;

public class SimControl{


	int targetLaps;

	//Position format is [Top/Bottom, Left/Right]
	String[] endLoc;

	//Position order is [Top-Left, Top-Right, Bottom-Right, Bottom-Left]
	int[] locVisits = {0,0,0,0};

	boolean simComplete = false;

	int rPP, pulseCounter, gapCounter;
	double rPPFract;
	//rPP = refreshes per pulse

	SimCarControl carControl;

	int FW, BW, NOPOW, STRGHT, L, R;
	String powerInfo;


	//Constructor
	public SimControl(String[] loc, int numLaps){

		targetLaps = numLaps;
		endLoc = loc;
		getStaticControls();
		carControl = new SimCarControl();

	}

	//Set control constants
	private void getStaticControls(){

		FW = StaticControls.FORWARD;
		BW = StaticControls.BACKWARD;
		NOPOW = StaticControls.NOPOW;
		STRGHT = StaticControls.STRAIGHT;
		L = StaticControls.LEFT;
		R = StaticControls.RIGHT;

	}

	//Initialise new action
	public void initialiseAction(SimAction action){

		rPP = action.getCompleteRefs();
		rPPFract = action.getFractRef();
		pulseCounter = 0;
		gapCounter = rPP;

	}

	//Run the simulation by moving instruction/action on by one increment/refresh
	public void runSim(SimCar car, SimAction action, boolean driveCar){

		String[] oldLocation = getLocation(car);

		switch(action.type) {
		//Type Key: 1:F, 2:FR, 3:FL, 4:B, 5:BR, 6:BL

			case 1:		gapCounter++;
						if(gapCounter == rPP && rPPFract > 0){
							powerInfo = "No Drive(frac refresh)";
							car = SimCalcs.forwards(car, action.fractThetaInc, action.pulseCoeff, rPPFract);
						}
						else if(gapCounter > rPP || (gapCounter == rPP && rPPFract == 0  && pulseCounter != action.pulses)){
							if(driveCar){
								carControl.drive(FW, STRGHT);
							}
							powerInfo = "Drive";
							gapCounter = 0;
							pulseCounter++;
							car = SimCalcs.forwards(car, action.thetaInc, action.pulseCoeff, 1);
						}
						else {
							if(gapCounter == 1 && driveCar){
								carControl.drive(NOPOW, STRGHT);
							}
							powerInfo = "No Drive";
							car = SimCalcs.forwards(car, action.thetaInc, action.pulseCoeff, 1);
						}
						break;

			case 2:		gapCounter++;
						if(gapCounter == rPP && rPPFract > 0){
							powerInfo = "No Drive(frac refresh)";
							car = SimCalcs.turnRight(car, action.fractThetaInc, action.pulseCoeff, rPPFract);
						}
						else if(gapCounter > rPP || (gapCounter == rPP && rPPFract == 0  && pulseCounter != action.pulses)){
							if(driveCar){
								carControl.drive(FW, R);
							}
							powerInfo = "Drive";
							gapCounter = 0;
							pulseCounter++;
							car = SimCalcs.turnRight(car, action.thetaInc, action.pulseCoeff, 1);
						}
						else {
							if(gapCounter == 1 && driveCar){
								carControl.drive(NOPOW, R);
							}
							powerInfo = "No Drive";
							car = SimCalcs.turnRight(car, action.thetaInc, action.pulseCoeff, 1);
						}
						break;

			case 3:		gapCounter++;
						if(gapCounter == rPP && rPPFract > 0){
							powerInfo = "No Drive(frac refresh)";
							car = SimCalcs.turnLeft(car, action.fractThetaInc, action.pulseCoeff, rPPFract);
						}
						else if(gapCounter > rPP || (gapCounter == rPP && rPPFract == 0 && pulseCounter != action.pulses)){
							if(driveCar){
								carControl.drive(FW, L);
							}
							powerInfo = "Drive";
							gapCounter = 0;
							pulseCounter++;
							car = SimCalcs.turnLeft(car, action.thetaInc, action.pulseCoeff, 1);
						}
						else {
							if(gapCounter == 1 && driveCar){
								carControl.drive(NOPOW, L);
							}
							powerInfo = "No Drive";
							car = SimCalcs.turnLeft(car, action.thetaInc, action.pulseCoeff, 1);
						}

						break;

			case 4:		//DOESN'T OCCUR AT PRESENT
						gapCounter++;
						if(gapCounter == rPP && rPPFract > 0){
							powerInfo = "No Drive(frac refresh)";
							//refresh car position for display - ANIMATION FOR BACKWARDS MOTION NOT YET CODED
						}
						else if(gapCounter > rPP || (gapCounter == rPP && rPPFract == 0 && pulseCounter != action.pulses)){
							if(driveCar){
								carControl.drive(BW, STRGHT);
							}
							gapCounter = 0;
							pulseCounter++;
							//refresh car position for display - ANIMATION FOR BACKWARDS MOTION NOT YET CODED
						}
						else {
							if(gapCounter == 1 && driveCar){
								carControl.drive(NOPOW, STRGHT);
							}
							//refresh car position for display - ANIMATION FOR BACKWARDS MOTION NOT YET CODED
						}
						break;

			case 5:		//DOESN'T OCCUR AT PRESENT
						gapCounter++;
						if(gapCounter == rPP && rPPFract > 0){
							powerInfo = "No Drive(frac refresh)";
							//refresh car position for display - ANIMATION FOR BACKWARDS MOTION NOT YET CODED
						}
						else if(gapCounter > rPP || (gapCounter == rPP && rPPFract == 0 && pulseCounter != action.pulses)){
							if(driveCar){
								carControl.drive(BW, R);
							}
							gapCounter = 0;
							pulseCounter++;
							//refresh car position for display - ANIMATION FOR BACKWARDS MOTION NOT YET CODED
						}
						else {
							if(gapCounter == 1 && driveCar){
								carControl.drive(NOPOW, R);
							}
							//refresh car position for display - ANIMATION FOR BACKWARDS MOTION NOT YET CODED
						}
						break;

			case 6:		//DOESN'T OCCUR AT PRESENT
						gapCounter++;
						if(gapCounter == rPP && rPPFract > 0){
							powerInfo = "No Drive(frac refresh)";
							//refresh car position for display - ANIMATION FOR BACKWARDS MOTION NOT YET CODED
						}
						else if(gapCounter > rPP || (gapCounter == rPP && rPPFract == 0 && pulseCounter != action.pulses)){
							if(driveCar){
								carControl.drive(BW, L);
							}
							gapCounter = 0;
							pulseCounter++;
							//refresh car position for display - ANIMATION FOR BACKWARDS MOTION NOT YET CODED
						}
						else {
							if(gapCounter == 1 && driveCar){
								carControl.drive(NOPOW, L);
							}
							//refresh car position for display - ANIMATION FOR BACKWARDS MOTION NOT YET CODED
						}
						break;

			case 0:		//No more instructions
						break;

			default:	System.out.println("ERROR with input");
						break;

		}

		if((pulseCounter == action.pulses) && (gapCounter == rPP)){

			action.complete = true;

		}


		String[] location = getLocation(car);

		if ((location[0] != oldLocation[0]) || (location[1] != oldLocation[1])) {//i.e. area of track car currently is in has changed

			int locAddr = getLocAddress(location);
			locVisits[locAddr]++;

			//Test to see if sim is now complete. Test is carried out on point B (front-left corner) of car
			if ((location[0] == endLoc[0]) && (location[1] == endLoc[1]) && (locVisits[locAddr] > targetLaps)){

				simComplete = true;

			}

		}

	}

	//Get the location of the passed car in terms of track sectors
	private String[] getLocation(SimCar car){

		String[] result = new String[2];
		if(car.yCoords[1]<324){
			if (car.xCoords[1]<620){
				result[0] = "Top";
				result[1] = "Left";
			} else {
				result[0] = "Top";
				result[1] = "Right";
			}
		}
		else {
			if (car.xCoords[1]<620){
				result[0] = "Bottom";
				result[1] = "Left";
			} else {
				result[0] = "Bottom";
				result[1] = "Right";
			}
		}
		return result;
	}

	//Returns the corresponding address in posVisits array for the given location
	private int getLocAddress(String[] location){

		int result;

		if (location[0] == "Top"){
			if(location[1] == "Left"){
				result = 0;
			}
			else {
				result = 1;
			}
		} else {
			if (location[1] == "Right"){
				result = 2;
			}
			else {
				result = 3;
			}
		}

		return result;

	}

	//Get the lap number the car is currently on
	//N.B. As car is unlikely to return to exact starting position each lap, the first lap doesn't start until the car crosses
	//the threshold separating position it starts from (TL, TR etc.) and the first position it visits after that. If it hasn't
	//yet crossed this (invisible) line, 0 is returned, signalling the first lap has not yet begun.
	public int getCurrentLap(){

		int max = 0;

		for (int i=0; i<locVisits.length; i++){
			if (locVisits[i]>max){
				max = locVisits[i];
			}
		}

		return max;

	}

}