/**
 * SimNavigator.java
 * Currently imports instructions from a text file.
 * Could be used in future to determine next action required.
 *
 * Instruction file format is: dir(string) steer(string) pulses(int) pulseCoeff(double)
 *
 * Called by SimActionHandler.java, for use with Simulation.java.
 *
 * Modified By: Andrew Armstrong: renamed to SimNavigator from Navigator
 *
 * Created by: Chris Carter
 * Last Modified: 04.05.08
 *
 */

import java.io.*;

public class SimNavigator {

	StreamTokenizer instFile;

	//Open file
	public SimNavigator(String fileName, CarAIMain passedCarAIMain){
		try {
			instFile = new StreamTokenizer(new BufferedReader(new FileReader(fileName)));
		} catch (Exception e){
			System.err.println("SimNavigator: Unable to open instruction file. Error: " + e.toString());
			passedCarAIMain.log("SimNavigator: Unable to open instruction file.\nPlease move file to correct directory and restart app\nError: " + e.toString());
		}
	}

	//Get the next textual instruction (dir/steer)
	public String getInst(){

		String result;
		try {
			instFile.nextToken();
			if(instFile.ttype == instFile.TT_EOF){
				result = "EOF";
			}
			else{
				result = instFile.sval;
			}
		}
		catch (Exception e){
			System.err.println("SimNavigator: Unable to read from instruction file. Error: " + e.toString());
			result = "ERROR";
		}
		return result;
	}

	//Get the next numeric instruction (pulses/coeff)
	public double getAttribute(){

		double result;
		try {
			instFile.nextToken();
			result = instFile.nval;
		}
		catch (Exception e){
			System.err.println("SimNavigator: Unable to read from instruction file. Error: " + e.toString());
			result = 0.0;
		}
		return result;
	}

}