// This simply checks overlapping car co-ordinates/corners with the wall and obsticle intersections. It only
// does it on the simulation of course.

/**
 * SimTestCrash.java
 * Tests whether the car object has crashed.
 * Called by SimDraw.java, for use with CarAIMain.java.
 *
 * Car orientation:
 *
 *					A------B
 *            Rear  	     Front
 *					D------C
 *
 * Modified by Andrew Armstrong: renamed to SimTestCrash from TestCrash.
 *
 * Created by: Chris Carter
 * Last Modified: 04.05.08
 *
 */

import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;

public abstract class SimTestCrash {

	static String[] cornerLabels = {"A","B","C","D"};
	static String[] wallLabels = {"Outer Wall", "Small Wall", "Centre Wall"};

	static JScrollBar vScrollBar;

	//Test to see if car in passed position has crashed, if so return true else return false.
	public static boolean testPos(SimTrackArea[] track, CarAIMain passedCarAIMain, SimCar car){

		boolean result = false;
		int i = 0;

		while (!result && i<track.length){

			int j=0;
			while (!result && j<car.xCoords.length){
				if(track[i].DRIVEABLE){
					result = testOuter(i, j, track[i], car, passedCarAIMain);
				}
				else {
					result = testInner(i, j, track[i], car, passedCarAIMain);
				}
				j++;
			}
			//Test to see if sides have car have clipped corner of a wall
			if(!result && car.carPoly.intersects(track[i]) && !track[i].DRIVEABLE){

				passedCarAIMain.log("Car has clipped "+wallLabels[i]);
				result = true;

			}
			i++;
		}
		return result;
	}

	//Test Corners for collision with Outer Walls
	private static boolean testOuter(int i, int j, SimTrackArea area, SimCar car, CarAIMain passedCarAIMain){

		boolean result = false;
		if (!area.contains(car.xCoords[j],car.yCoords[j])){
			passedCarAIMain.log("Car has hit the "+wallLabels[i]+" (Point "+cornerLabels[j]+")");
			result = true;
		}
		return result;
	}

	//Test Corners for collision with Inner Walls
	private static boolean testInner(int i, int j, SimTrackArea area, SimCar car, CarAIMain passedCarAIMain){

		boolean result = false;
		if (area.contains(car.xCoords[j],car.yCoords[j])){
			passedCarAIMain.log("Car has hit the "+wallLabels[i]+" (Point "+cornerLabels[j]+")");
			result = true;
		}
		return result;
	}

}