/**
 * Utility.java
 * 
 * Any utility-like general function which is used in more then one class can be referenced from here. Since it is statically bound it requires
 * no creation to work.
 *
 * Created by: Andrew Armstrong
 * Last Modified: 04/05/08
 *
 */

// StringTokenizer
import java.util.*;
import java.lang.Math.*;

public class Utility {
	
	/*************************
		PUBLIC FUNCTIONS 
	*************************/

	/**
	* formatFileName method modified from removeSpaces method
	* Written by Ral Gagnon (2005)
	* Copied from http://64.18.163.122/rgagnon/javadetails/java-0352.html
	* Accessed: 21.04.07 by Chris Carter
	* Altered 2008: Andrew Armstrong, no space removal, colons to hyphens.
	*/
	public static String formatFileName(String s) {
		StringTokenizer st = new StringTokenizer(s,":",false);
		String t="";
		while (st.hasMoreElements()){
			t += st.nextElement();
			t += "-";
		}
		return t;
	}

	// calculates the angle between two coordinates in degrees.
	// Sourced from http://www.uk-dave.com/bytes/java/angle_between_points.html
    public static double calcAngle(Coordinate coords1, Coordinate coords2) { return calcAngle(coords1.x, coords1.y, coords2.x, coords2.y); }
    public static double calcAngle(float x1, float y1, float x2, float y2)
    {
        float dx = x2-x1;
        float dy = y2-y1;
        double angle=0.0d;
 
        // Calculate angle
        if (dx == 0.0)
        {
            if (dy == 0.0)
                angle = 0.0;
            else if (dy > 0.0)
                angle = Math.PI / 2.0;
            else
                angle = Math.PI * 3.0 / 2.0;
        }
        else if (dy == 0.0)
        {
            if  (dx > 0.0)
                angle = 0.0;
            else
                angle = Math.PI;
        }
        else
        {
            if  (dx < 0.0)
                angle = Math.atan(dy/dx) + Math.PI;
            else if (dy < 0.0)
                angle = Math.atan(dy/dx) + (2*Math.PI);
            else
                angle = Math.atan(dy/dx);
        }
 
        // Convert to degrees
        angle = angle * 180 / Math.PI;
 
        // Return
        return angle;
    }
	
	// Saves writing this out each time, a simple function to limit an integer
	public static int getBoundriedInt(int value, int min, int max)
	{
		if(value > max) return max;
		if(value < min) return min;
		return value;
	}
	
	// Gets the distance between 2 different points
	public static double getDistanceBetween(Coordinate one, Coordinate two) { return getDistanceBetween(one.x, one.y, two.x, two.y); }
	public static double getDistanceBetween(int x1, int y1, int x2, int y2)
	{
		double xd = x2-x1;
		double yd = y2-y1;
		return Math.abs( Math.sqrt(xd * xd + yd * yd) );
	}

	// Adds up the distances in the Coordiate array "coordArray" from entry startIndex to entry endIndex
	public static double getTotalDistancesFromArray(int startIndex, int endIndex, Coordinate[] coordArray)
	{
		double total = 0.0;
		for(int i = startIndex; i < endIndex; i++)
		{
			total += getDistanceBetween(coordArray[i].x, coordArray[i].y, coordArray[i + 1].x, coordArray[i + 1].y);
		}
		return total;
	}
	
	// Project a point from a "line" between i and j forwards a certain amount of pixels, return the coordinate it is. Minimum and maximum points also available.
	// IE: Line between i and j results in return point r:
	// 	i ----------- j ----------- r
	public static Coordinate getProjectedPoint(Coordinate i, Coordinate j, int pixels, int minX, int maxX, int minY, int maxY)
	{
		Coordinate returnCoordinate = new Coordinate();
		
		// End up with an angle 0-259.999 (need radians for the cos / sin stuff, so convert too)
		double angle = Math.toRadians(calcAngle(i.x, i.y, j.x, j.y));

		// Working...bah, maths
		// If we are going 5 "forwards", and angle is 0 (ie; on the X plane), we have a case of simply adding 5 to x, 0 to y.
		// 0 is "up" x, so +5 to x, 0 change to y
		// 90 is "down" y, so +5 to y, 0 change to x
		// 180 is -5 from x, 0 change to y.
		// 270 is "up" y so -5 from y, 0 change to x
		
		// cos: 0 is 1, 90 is 0, 180 is -1, 270 is 0, 360 is 1
		// 	-> x
		// sin; 0 is 0, 90 is 1, 180 is 0, 270 is -1 360 is 0
		//	-> y

		// Get the new points using above stuff
		// Math.sin and Math.cos use radians, not degrees
		returnCoordinate.x = getBoundriedInt(j.x + (int)(pixels * Math.cos(angle)), minX, maxX);
		returnCoordinate.y = getBoundriedInt(j.y + (int)(pixels * Math.sin(angle)), minY, maxY);

		return returnCoordinate;
	}
	// As above but using a already calculated angle
	public static Coordinate getProjectedPoint(Coordinate j, double angle, int pixels, int minX, int maxX, int minY, int maxY)
	{
		Coordinate returnCoordinate = new Coordinate();

		// End up with an angle 0-259.999 (need radians for the cos / sin stuff, so convert too)
		angle = Math.toRadians(angle);

		// Get the new points using above stuff
		// Math.sin and Math.cos use radians, not degrees
		returnCoordinate.x = getBoundriedInt(j.x + (int)(pixels * Math.cos(angle)), minX, maxX);
		returnCoordinate.y = getBoundriedInt(j.y + (int)(pixels * Math.sin(angle)), minY, maxY);

		return returnCoordinate;
	}
	
	// Random integer between min and max
	public static int randomInt(int min, int max)
	{
		double rand = Math.random();
		return min + (int)((max - min)* rand);
	}
	
	// Get a precent of this integer
	public static int getPercent(int input, int percent)
	{
		return (int) ( (double)input * ((double)percent/100));
	}
}