package ppv;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
  * BitField.java
  *
  * Defines bit fields for each pin on the parallel port
  *
  * Code originally by Soft Collection as part of Parallel Port Viewer
  * Downloaded from: http://www.planet-source-code.com/vb/scripts/ShowCode.asp?txtCodeId=4309&lngWId=2
  * Originally modified by: Chris Carter
  *
  * Later modified by: Andrew Armstrong (formatting mainly)
  * Last Modified: 01.04.08
  *
  */
public class BitField extends JTextField {

	// Variables
	private static final int howManyFieldsAre=8;//18;
	private static  int fieldWidth=12; //Width of the field image
	private static  int fieldHeight=15; //Height of the field image
	public static boolean fieldsOnPortViewerArrayValue[] = new boolean[howManyFieldsAre];
	public static boolean fieldsOnPortViewerArrayIsActive[] = new boolean[howManyFieldsAre];
	private int thisFieldNumber;
	private UpdateField updateField = new UpdateField();
	private static final int fieldUpdateDelay = 100;

	private final static long serialVersionUID = 1L;

	// Constructor
	public BitField(int coordinateXOnParent, int coordinateYOnParent, int thisFieldNumber, String toolTip){
		InitField();
		setThisFieldNumber(thisFieldNumber);
		setFieldLocation(coordinateXOnParent,coordinateYOnParent);
		setToolTipText(toolTip);
	}
	private void InitField(){
		setPreferredSize(new Dimension(fieldWidth,fieldHeight));
		setOpaque(false);
		// addMouseListener(new MouseFunctions());
		setDisabledTextColor(Color.BLACK);
		setFont(new Font("Arial",Font.BOLD,12));
		setEditable(false);
		setFieldValue();
		updateField.start();
	}

	// AA: Appears this is unused - previously used to do the activation?
	/* private class MouseFunctions extends MouseAdapter{
	public void mousePressed(MouseEvent e){
		if (e.getButton()==MouseEvent.BUTTON1){
			if (fieldsOnPortViewerArrayIsActive[thisFieldNumber]==true) fieldsOnPortViewerArrayValue[thisFieldNumber]=(!fieldsOnPortViewerArrayValue[thisFieldNumber]);
		}
	}
	}*/

	public void setThisFieldNumber(int thisFieldNumber){
		this.thisFieldNumber=thisFieldNumber;
	}
	
	public void setFieldLocation(int coordinateXOnParent,int coordinateYOnParent){
		setBounds(coordinateXOnParent,coordinateYOnParent,fieldWidth,fieldHeight);
	}
	
	public void setFieldValue(){
		switch(thisFieldNumber){
			/*case 1:*/case 11:case 14:case 17: setText(""+((fieldsOnPortViewerArrayValue[thisFieldNumber]==false)?1:0)); break;
			default: setText(""+((fieldsOnPortViewerArrayValue[thisFieldNumber]==false)?0:1)); break;
		}
	}

	// Thread to update things without interrupting the GUI
	private class UpdateField extends Thread{
		boolean fieldTemp;
		public void run(){
			fieldTemp = fieldsOnPortViewerArrayValue[thisFieldNumber];
			while(true){
				if (fieldTemp != fieldsOnPortViewerArrayValue[thisFieldNumber]){
					setFieldValue();
					fieldTemp = fieldsOnPortViewerArrayValue[thisFieldNumber];
				}
				if (thisFieldNumber==0){
					if (fieldsOnPortViewerArrayValue[0]==false) dataOutActives();
					if (fieldsOnPortViewerArrayValue[0]==true) dataInActives();
				}
				try{Thread.sleep(fieldUpdateDelay);}catch(InterruptedException e){}
			}
		}
	}

	private void dataInActives(){
		fieldsOnPortViewerArrayIsActive[0] = true;
		fieldsOnPortViewerArrayIsActive[1] = true;
		fieldsOnPortViewerArrayIsActive[2] = false;
		fieldsOnPortViewerArrayIsActive[3] = false;
		fieldsOnPortViewerArrayIsActive[4] = false;
		fieldsOnPortViewerArrayIsActive[5] = false;
		fieldsOnPortViewerArrayIsActive[6] = false;
		fieldsOnPortViewerArrayIsActive[7] = false;
		/* fieldsOnPortViewerArrayIsActive[8] = false;
		fieldsOnPortViewerArrayIsActive[9] = false;
		fieldsOnPortViewerArrayIsActive[10] = false;
		fieldsOnPortViewerArrayIsActive[11] = false;
		fieldsOnPortViewerArrayIsActive[12] = false;
		fieldsOnPortViewerArrayIsActive[13] = false;
		fieldsOnPortViewerArrayIsActive[14] = true;
		fieldsOnPortViewerArrayIsActive[15] = false;
		fieldsOnPortViewerArrayIsActive[16] = true;
		fieldsOnPortViewerArrayIsActive[17] = true;*/
	}
	private void dataOutActives(){
		fieldsOnPortViewerArrayIsActive[0] = true;
		fieldsOnPortViewerArrayIsActive[1] = true;
		fieldsOnPortViewerArrayIsActive[2] = true;
		fieldsOnPortViewerArrayIsActive[3] = true;
		fieldsOnPortViewerArrayIsActive[4] = true;
		fieldsOnPortViewerArrayIsActive[5] = true;
		fieldsOnPortViewerArrayIsActive[6] = true;
		fieldsOnPortViewerArrayIsActive[7] = true;
		/* fieldsOnPortViewerArrayIsActive[8] = true;
		fieldsOnPortViewerArrayIsActive[9] = true;
		fieldsOnPortViewerArrayIsActive[10] = false;
		fieldsOnPortViewerArrayIsActive[11] = false;
		fieldsOnPortViewerArrayIsActive[12] = false;
		fieldsOnPortViewerArrayIsActive[13] = false;
		fieldsOnPortViewerArrayIsActive[14] = true;
		fieldsOnPortViewerArrayIsActive[15] = false;
		fieldsOnPortViewerArrayIsActive[16] = true;
		fieldsOnPortViewerArrayIsActive[17] = true;*/
	}

}